# views.py
import secrets
import uuid
from django.db.models import Q
from datetime import timedelta
from django.forms import ValidationError
from django.core.validators import validate_email
import requests
from rest_framework import status
from rest_framework import status, generics
from rest_framework.response import Response
from rest_framework.views import APIView
from django.contrib.auth import authenticate
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework_simplejwt.tokens import RefreshToken
from rest_framework_simplejwt.authentication import JWTAuthentication
from rest_framework_simplejwt.tokens import BlacklistedToken, OutstandingToken
from django.utils import timezone
from crea.settings import verify_google_oauth_token
from crea_app.storages import UthoStorage
from .utils import send_otp_via_fast2sms, send_otp_via_sms
from .models import ArtiztNetworkAccNumber, City, Country, CustomUser, DeletedAccount, DeviceList, OTPVerification, PrivacySettings, State, StateShortNames, UserSession
from .serializers import CitySerializer, PrivacySettingsSerializer, StateSerializer, StateShortNamesSerializer, UserSerializer,RegisterSerializer, OTPSerializer
from django.core.mail import send_mail
from .serializers import RegisterSerializer
from drf_yasg.utils import swagger_auto_schema
import random
from rest_framework import generics
from .models import PasswordResetRequest
from .serializers import ResetPasswordSerializer, ChangePasswordSerializer
from django.contrib.auth import get_user_model
from .models import CustomUser
from django.conf import settings
from django.core.mail import send_mail
from django.contrib.auth import get_user_model
import random
from rest_framework_simplejwt.token_blacklist.models import OutstandingToken
from .serializers import LogoutSerializer, UserSerializer, RegisterSerializer, OTPSerializer, ResetPasswordSerializer, ChangePasswordSerializer, OnboardingImageSerializer
from .models import CustomUser, OTPVerification, UserSession, OnboardingImage, PasswordResetRequest
from drf_yasg.utils import swagger_auto_schema
from django.contrib.auth.hashers import check_password
from rest_framework_simplejwt.token_blacklist.models import BlacklistedToken
from rest_framework_simplejwt.token_blacklist.models import OutstandingToken
from firebase_admin import auth as firebase_auth
from .utils import send_otp_via_sms  
from dotenv import load_dotenv
from random import randint
from django.core.validators import validate_email
from django.core.exceptions import ValidationError
import firebase_admin
from firebase_admin import credentials, auth as firebase_auth
from firebase_admin.exceptions import FirebaseError
from django.utils.timezone import now
from django.contrib.auth import logout as django_logout
from django.http import JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views import View
from src.ChatTokenBuilder2 import ChatTokenBuilder 
from decouple import config

User = get_user_model()

APP_ID = config('APP_ID')
APP_CERTIFICATE = config('APP_CERTIFICATE')

class OnboardingAPIView(APIView):
    permission_classes = [AllowAny]

    def get(self, request, pk=None):
        if pk:
            try:
                onboarding_image = OnboardingImage.objects.get(pk=pk)
            except OnboardingImage.DoesNotExist:
                return Response({
                    "status": {
                        "type": "error",
                        "code": 404,
                        "error": True
                    },
                    "message": "Onboarding image not found."
                }, status=status.HTTP_404_NOT_FOUND)

            serializer = OnboardingImageSerializer(onboarding_image)
        else:
            onboarding_images = OnboardingImage.objects.all()
            serializer = OnboardingImageSerializer(onboarding_images, many=True)

        return Response({
            "status": {
                "type": "success",
                "code": 200,
                "error": False,
                "message": "Onboarding images retrieved successfully."
            },
            "data": serializer.data
        }, status=status.HTTP_200_OK)

    def post(self, request):
        # Check if the 'image' file is in the request's FILES
        if 'image' not in request.FILES:
            return Response({
                "status": {
                    "type": "error",
                    "code": 400,
                    "error": True
                },
                "message": "No image file provided."
            }, status=status.HTTP_400_BAD_REQUEST)

        # Initialize the serializer with request data
        serializer = OnboardingImageSerializer(data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": {
                    "type": "success",
                    "code": 201,
                    "error": False,
                    "message": "Onboarding image created successfully."
                },
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)

        return Response({
            "status": {
                "type": "error",
                "code": 400,
                "error": True
            },
            "message": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def put(self, request, pk):
        try:
            onboarding_image = OnboardingImage.objects.get(pk=pk)
        except OnboardingImage.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "code": 404,
                    "error": True
                },
                "message": "Onboarding image not found."
            }, status=status.HTTP_404_NOT_FOUND)

        # Update with request data
        serializer = OnboardingImageSerializer(onboarding_image, data=request.data, partial=True)
        if serializer.is_valid():
            serializer.save()
            return Response({
                "status": {
                    "type": "success",
                    "code": 200,
                    "error": False,
                    "message": "Onboarding image updated successfully."
                },
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "status": {
                "type": "error",
                "code": 400,
                "error": True
            },
            "message": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, pk):
        try:
            onboarding_image = OnboardingImage.objects.get(pk=pk)
            onboarding_image.delete()
            return Response({
                "status": {
                    "type": "success",
                    "code": 204,
                    "error": False,
                    "message": "Onboarding image deleted successfully."
                }
            }, status=status.HTTP_204_NO_CONTENT)
        except OnboardingImage.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "code": 404,
                    "error": True
                },
                "message": "Onboarding image not found."
            }, status=status.HTTP_404_NOT_FOUND)


class RegisterView(APIView):
    permission_classes = [AllowAny]

    @swagger_auto_schema(request_body=RegisterSerializer)
    def post(self, request):
        serializer = RegisterSerializer(data=request.data)
        if serializer.is_valid():
            user = serializer.save()
            try:
                # Determine if the user identifier is an email or mobile number
                is_email = False
                try:
                    validate_email(user.mobile_or_email)
                    is_email = True
                except ValidationError:
                    pass

                # Generate OTP and save OTP verification record
                otp = str(random.randint(100000, 999999))
                print(otp)
                otp_verification = OTPVerification.objects.create(
                    user=user,
                    otp=otp,
                    otp_expires_at=timezone.now() + timedelta(minutes=10)
                )

                # Send OTP via email or SMS
                if is_email:
                    send_mail(
                        'Your OTP Code',
                        f'Your OTP code is {otp}. It is valid for 10 minutes.',
                        settings.EMAIL_HOST_USER,
                        [user.mobile_or_email],
                        fail_silently=False,
                    )
                    success_message = "OTP sent to your email address"
                else:
                    # Attempt to send OTP via SMS
                    # response_data = send_otp_via_sms(user.mobile_or_email, otp)
                    response_data = send_otp_via_fast2sms(user.mobile_or_email, otp)
                    user.mobile_number = user.mobile_or_email
                    # user.mobile_or_email = None
                    user.save()
                    if response_data.get('status') == 'success':
                        success_message = "OTP sent to your mobile number"
                    else:
                        return Response({
                            "status": {
                                "type": "failed",
                                "message": "Failed to send OTP via SMS. Please try again.",
                                "code": 500,
                                "error": True
                            }
                        }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

            except Exception as e:
                # Handle unexpected exceptions
                return Response({
                    "status": {
                        "type": "failed",
                        "message": "An error occurred while sending OTP. Please try again.",
                        "code": 500,
                        "error": True
                    }
                }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

            # Return success response with OTP and expiration details
            return Response({
                "status": {
                    "id":user.id,
                    "type": "success",
                    "message": success_message,
                    "code": 200,
                    "error": False,
                    "otp":otp
                },
                "data": [{
                    "otp": otp,  # Include OTP regardless of email or SMS
                    "otp_expires_at": otp_verification.otp_expires_at
                }]
            }, status=status.HTTP_200_OK)

        # Return validation errors in a consistent format
        return Response({
            "status": {
                "type": "failed",
                "message": "Mobile / Email address already Exist",
                "code": 400,
                "error": True
            },
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)
    
if not firebase_admin._apps:
    cred = credentials.Certificate("crea/serviceAccountKey.json")
    firebase_admin.initialize_app(cred)

from django.apps import apps
class LoginView(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        login_method = request.data.get('login_method')  # "1" for traditional, "2" for Google, "3" for Apple
        mobile_or_email = request.data.get('mobile_or_email')
        password = request.data.get('password')
        google_token = request.data.get('google_token')
        apple_token = request.data.get('apple_token')
        fullname = request.data.get('fullname')
        fcm_token = request.data.get('fcm_token')
        # print(google_token)
        print(fcm_token)

        if login_method not in ["1", "2", "3"]:
            return Response({
                "status": {
                    "type": "failed",
                    "message": "Invalid login method",
                    "code": 400,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            if login_method == "1":
                return self.handle_traditional_login(mobile_or_email, password)
            elif login_method == "2":
                return self.handle_google_login(google_token, fullname)
            elif login_method == "3":
                return self.handle_apple_login(apple_token)
        except Exception as e:
            return Response({
                "status": {
                    "type": "failed",
                    "message": f"An unexpected error occurred: {str(e)}",
                    "code": 500,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def handle_traditional_login(self, mobile_or_email, password):
        try:
            if '@' in mobile_or_email:
                user = CustomUser.objects.get(mobile_or_email=mobile_or_email)
            else:
                user = CustomUser.objects.get(mobile_number = mobile_or_email)
            if not user.is_active:
                return Response({
                    "status": {
                        "type": "failed",
                        "message": "Account has been deactivated. Please contact support for assistance.",
                        "code": 403,
                        "error": True
                    },
                    "data": []
                }, status=status.HTTP_403_FORBIDDEN)

            if not user.is_verified and not user.is_mobile_verified:
                return Response({
                    "status": {
                        "type": "failed",
                        "message": "Account not verified. Please verify your OTP.",
                        "code": 403,
                        "error": True
                    },
                    "data": []
                }, status=status.HTTP_403_FORBIDDEN)

            if user.check_password(password):
                fcm_token = self.request.data.get("fcm_token")
                if fcm_token:
                    user.fcm_token = fcm_token
                    user.save()

                refresh = RefreshToken.for_user(user)
                return self.generate_response(user, refresh, "1")
            else:
                return Response({
                    "status": {
                        "type": "failed",
                        "message": "Invalid password",
                        "code": 401,
                        "error": True
                    },
                    "data": []
                }, status=status.HTTP_401_UNAUTHORIZED)

        except CustomUser.DoesNotExist:
            return Response({
                "status": {
                    "type": "failed",
                    "message": "User does not exist",
                    "code": 404,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_404_NOT_FOUND)
        
    def handle_google_login(self, google_token, fullname):
        print("Full name in handle_google_login", fullname)
        if not google_token:
            return Response({
                "status": {
                    "type": "failed",
                    "message": "Google token must not be empty.",
                    "code": 400,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            # Verify the Google ID token using Firebase Admin SDK
            decoded_token = verify_google_oauth_token(google_token)
            if not decoded_token:
                return Response({
                    "status": {
                        "type": "failed",
                        "message": "Invalid or expired Google token.",
                        "code": 401,
                        "error": True
                    },
                    "data": []
                }, status=status.HTTP_401_UNAUTHORIZED)

            email = decoded_token.get('email')
            print(email)
            # Retrieve or create the user
            user, created = CustomUser.objects.get_or_create(
                mobile_or_email=email,
                # full_name = fullname,
                defaults ={
                "user_status": "active",
                "full_name": fullname,
                "login_method": "2"
                }
                # defaults={"user_status": "active"}
            )
            if not created and user.full_name != fullname:
                # Update the full_name if it has changed
                user.full_name = fullname
                user.save()

            # If the user is newly created, you can set additional fields or perform any initialization
            if created:
                base_username = f"@{fullname.replace(' ', '').lower()}"
                unique_username = base_username

                # Ensure the username is unique
                if CustomUser.objects.filter(username=unique_username).exists():
                    random_number = random.randint(1000, 9999)
                    unique_username = f"{base_username}{random_number}"

                user.username = unique_username
                user.save()
                print(f"Generated unique username: {unique_username}")
            fcm_token = self.request.data.get("fcm_token")
            if fcm_token:
                user.fcm_token = fcm_token
                user.save()

            expire = 315360000
            agora_username = user.username.lstrip('@')
            user_token = ChatTokenBuilder.build_user_token(APP_ID, APP_CERTIFICATE, agora_username, expire)

            # If user is newly created, register them with Agora
            if created:
                app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, expire)

                agora_payload = {
                    "username": agora_username
                }

                try:
                    agora_response = requests.post(
                        settings.AGORA_BASE_URL + "/users",
                        json=agora_payload,
                        headers={"Content-Type": "application/json", "Authorization": f"Bearer {app_token}"}
                    )

                    if agora_response.status_code != 200:
                        raise Exception(f"Agora registration failed: {agora_response.json()}")

                    agora_data = agora_response.json()
                    print(agora_data)
                    agora_uuid = agora_data.get("entities", [{}])[0].get("uuid")

                    if not agora_uuid:
                        raise Exception("UUID not found in Agora response")

                    # Store UUID and token in user model
                    user.agora_uid = agora_uuid
                    user.agora_token = user_token
                    user.agora_username = agora_username
                    user.save()

                except Exception as e:
                    return Response({
                        "status": "error",
                        "message": "Failed to register user with Agora",
                        "details": str(e),
                        "code": 500
                    }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

            # Only update the token for existing users
            else:
                user.agora_token = user_token
                user.save()
            refresh = RefreshToken.for_user(user)
            return self.generate_response(user, refresh, "2")

        except firebase_auth.FirebaseError as e:
            return Response({
                "status": {
                    "type": "failed",
                    "message": f"Google token verification failed: {str(e)}",
                    "code": 401,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_401_UNAUTHORIZED)
        except Exception as e:
            return Response({
                "status": {
                    "type": "failed",
                    "message": f"An unexpected error occurred: {str(e)}",
                    "code": 500,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def handle_apple_login(self, apple_token):
        if not apple_token:
            return Response({
                "status": {
                    "type": "failed",
                    "message": "Apple token must not be empty.",
                    "code": 400,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            decoded_token = verify_google_oauth_token(apple_token)
            if not decoded_token:
                return Response({
                    "status": {
                        "type": "failed",
                        "message": "Invalid or expired Apple token.",
                        "code": 401,
                        "error": True
                    },
                    "data": []
                }, status=status.HTTP_401_UNAUTHORIZED)

            email = decoded_token.get('email')
            user, created = CustomUser.objects.get_or_create(
                mobile_or_email=email,
                defaults={"user_status": "active"}
            )

            if not user.is_verified:
                return Response({
                    "status": {
                        "type": "failed",
                        "message": "Account not verified. Please verify your OTP.",
                        "code": 403,
                        "error": True
                    },
                    "data": []
                }, status=status.HTTP_403_FORBIDDEN)

            refresh = RefreshToken.for_user(user)
            return self.generate_response(user, refresh, "3")

        except Exception as e:
            return Response({
                "status": {
                    "type": "failed",
                    "message": f"Invalid Apple token: {str(e)}",
                    "code": 401,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_401_UNAUTHORIZED)
    # def check_verification_status(self, user):
    #     print("check_verification_status",user.id)
    #     # Dynamically import models from the userprofile_app
    #     AadharVerification = apps.get_model('userprofile_app', 'AadharVerification')
    #     PassportVerification = apps.get_model('userprofile_app', 'PassportVerification')
    #     DLVerification = apps.get_model('userprofile_app', 'DLVerification')

    #     # Check if any document is verified for the user
    #     is_verified = (
    #         AadharVerification.objects.filter(mobile_or_email=user).exists() or
    #         PassportVerification.objects.filter(mobile_or_email=user).exists() or
    #         DLVerification.objects.filter(mobile_or_email=user).exists()
    #     )

    #     return is_verified
    def check_verification_status(self, user):
        ConfirmKYC = apps.get_model('userprofile_app', 'ConfirmKYC')
        is_verified = ConfirmKYC.objects.filter(user_id=user.id).exists()

        return is_verified
    
    def check_account_verified(self, user):
        Profile = apps.get_model('userprofile_app', 'Profile')

        profile = Profile.objects.filter(user=user.id).first()

        if not profile:
            return False
        
        required_fields = [
            profile.selected_role,
            profile.selected_primary_industry,
            profile.selected_primary_skill,
            profile.selected_industries.exists(),
            profile.selected_skills.exists(),
        ]

        if any(field is None or field == '' for field in required_fields):
            return False

        return True  # All required fields are filled

    def generate_response(self, user, refresh, login_method):
        user_verified_status = self.check_verification_status(user)
        account_verified_status = self.check_account_verified(user)

        device_name = self.request.data.get("device_name", "Unknown Device")
        device_ip = self.request.data.get("device_ip", "Unknown IP")
        # print(device_ip)
        # device_ip = self.request.META.get("REMOTE_ADDR", "Unknown IP")

        session_id = str(refresh)
        existing_device = DeviceList.objects.filter(
            user=user,
            device_name=device_name,
            device_ip=device_ip,
            status="active"
        ).first()

        if existing_device:
            # Update the existing record's login time
            existing_device.login_time = now()
            existing_device.save()
        else:
            # Store device login details in DeviceList model
            DeviceList.objects.create(
                user=user,
                session_id=session_id,
                device_name=device_name,
                login_time=now(),
                device_ip=device_ip,
                status="active"
            )
        is_email = "@" in user.mobile_or_email

        # Add email_verified and mobile_verified keys
        email_verified = is_email
        mobile_verified = not is_email
        email_verified = str(email_verified)
        mobile_verified =str(mobile_verified)
        print(str(refresh.access_token))

        return Response({
            "status": {
                "type": "success",
                "message": "Logged in successfully",
                "code": 200,
                "error": False
            },
            "data": [{
                "email": user.mobile_or_email,
                "user_id": user.id,
                "Username":user.username,
                "Full_name":user.full_name,
                "email_verified": email_verified,
                "mobile_verified": mobile_verified,
                "user_verified": "true" if user_verified_status else "false",
                "account_status": "true" if account_verified_status else "false",
                "user_status": user.user_status,
                "login_method": login_method,
                "access_token": str(refresh.access_token),
                "refresh": str(refresh),
                "session_id": session_id
                # "tokens": {
                #     "refresh": str(refresh),
                #     "access": str(refresh.access_token),
                # }
            }]
        }, status=status.HTTP_200_OK)

class VerifyOTPView(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        mobile_or_email = request.data.get('mobile_or_email')
        otp_received = request.data.get('otp')
        action = request.data.get('action')  # This can be 'reset_password' or 'login'

        User = get_user_model()
        print(mobile_or_email)

        try:
            if '@' in mobile_or_email:
                user = User.objects.get(mobile_or_email=mobile_or_email)
            else:
                user = User.objects.get(mobile_number = mobile_or_email)
            otp_verification = OTPVerification.objects.get(user=user, otp=otp_received)

            if timezone.now() > otp_verification.otp_expires_at:
                return Response({
                    "status": {
                        "type": "error",
                        "message": "OTP has expired",
                        "code": 400,
                        "error": True
                    }
                }, status=status.HTTP_400_BAD_REQUEST)
            
            print(user)

            if not user.username:
                otp_verification.is_verified = True
                otp_verification.save()
                full_name = user.full_name  # Get the full_name value
                base_username = f"@{full_name.replace(' ', '').lower()}"  # Remove spaces and lowercase
                unique_username = base_username

                # Generate a random 4-digit number
                while User.objects.filter(username=unique_username).exists():
                    random_number = random.randint(1000, 9999)
                    unique_username = f"{base_username}{random_number}"

                # Update the user's `username` field with the generated username
                user.username = unique_username
                user.save()

            if '@' in mobile_or_email:
                login_method = "email"
                user.is_verified = True
                user.save()
            elif mobile_or_email.isdigit():
                login_method = "mobile"
                user.is_mobile_verified = True
                user.save()
            else:
                login_method = "google"  # Assuming 'google' for other cases

            print(user.agora_username)

            if not user.agora_username:
                # expire = 3600
                app_token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, settings.AGORA_EXPIRE_TIME)
                print(app_token)

                agora_username = user.username.lstrip('@')

                print(agora_username)
                agora_payload = {
                    "username": agora_username
                }

                try:
                    # Agora Registration
                    agora_response = requests.post(
                        settings.AGORA_BASE_URL + "/users",
                        json=agora_payload,
                        headers={"Content-Type": "application/json", "Authorization": f"Bearer {app_token}"}
                    )
                    print(agora_response)

                    if agora_response.status_code != 200:
                        raise Exception(f"Agora registration failed: {agora_response.json()}")

                    agora_data = agora_response.json()
                    print(agora_data)
                    agora_uuid = agora_data.get("entities", [{}])[0].get("uuid")
                    # print(agora_uuid)
                    # user_token = ChatTokenBuilder.build_user_token(settings.AGORA_APP_ID, settings.AGORA_APP_CERTIFICATE, agora_username, settings.AGORA_EXPIRE_TIME)
                    user_token = ChatTokenBuilder.build_user_token(APP_ID, APP_CERTIFICATE, agora_username, config('AGORA_EXPIRE_TIME'))

                    if not agora_uuid:
                        raise Exception("UUID not found in Agora response")

                    # Store UUID in user model
                    user.agora_uid = agora_uuid
                    user.agora_token = user_token
                    user.agora_username = agora_username
                    user.save()
                
                except Exception as e:
                    return Response({
                        "status": "error",
                        "message": "Failed to register user with Agora",
                        "details": str(e),
                        "code": 500
                    }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
            # else:
            #     user_token = user.agora_token 
            

            success_message="OTP Verified Successfully and User rgistered Successfully"
            temp_access_token = secrets.token_urlsafe(32)
            refresh = RefreshToken.for_user(user)
            return Response({
                "status": {
                    "type": "success",
                    "message": success_message,
                    "code": 200,
                    "email": user.mobile_or_email,
                    "method":login_method, # 1- email 2- mobile 3. google
                    "user_id": user.id,
                    "full_name":user.full_name,
                    "temp_access_token": temp_access_token,
                    "access_token": str(refresh.access_token),
                    "refresh": str(refresh),
                    "code": 200,
                    "error": False,
                }, 
                "user": {
                    "email": user.mobile_or_email,
                    "user_id": user.id
                },
            }, status=status.HTTP_200_OK)

        except User.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "message": "User does not exist",
                    "code": 404,
                    "error": True
                }
            }, status=status.HTTP_404_NOT_FOUND)
        except OTPVerification.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "message": "Please enter an correct OTP",
                    "code": 400,
                    "error": True
                }
            }, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            import traceback
            print("Error Traceback:", traceback.format_exc())  # Logs the full error stack
            return Response({
                "status": {
                    "type": "error",
                    "message": f"Server Issue: {str(e)}",  # Show actual error
                    "code": 500,
                    "error": True
                }
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class ResendOTPView(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        mobile_or_email = request.data.get('mobile_or_email')

        User = get_user_model()

        try:
            if '@' in mobile_or_email:
                user = User.objects.get(mobile_or_email=mobile_or_email)
            else:
                user = User.objects.get(mobile_number=mobile_or_email)
                otp_verifications = OTPVerification.objects.filter(user=user)
                if otp_verifications.exists():
                    latest_otp = otp_verifications.order_by('-otp_expires_at').first()

                    # Check if the user is requesting OTP within 5 minutes
                    if latest_otp and latest_otp.otp_expires_at > timezone.now() and not latest_otp.is_verified:
                        time_since_last_request = timezone.now() - latest_otp.otp_expires_at
                        if time_since_last_request < timezone.timedelta(minutes=5):
                            return Response({
                                "status": {
                                    "type": "error",
                                    "message": "You can only request OTP for your mobile number once every 5 minutes.",
                                    "code": 429,
                                    "error": True
                                }
                            }, status=status.HTTP_429_TOO_MANY_REQUESTS)
            otp_verifications = OTPVerification.objects.filter(user=user)

            if otp_verifications.exists():
                latest_otp = otp_verifications.order_by('-otp_expires_at').first()
                otp = latest_otp.otp if latest_otp and not latest_otp.is_verified and latest_otp.otp_expires_at > timezone.now() else str(randint(100000, 999999))
                print(otp)
                if not latest_otp or latest_otp.is_verified or latest_otp.otp_expires_at <= timezone.now():
                    OTPVerification.objects.create(user=user, otp=otp)

                if '@' in mobile_or_email:
                    send_mail(
                        'Your OTP Code',
                        f'Your OTP code is {otp}. It is valid for 10 minutes.',
                        settings.EMAIL_HOST_USER,
                        [user.mobile_or_email],
                        fail_silently=False,
                    )
                    response_message = "OTP sent successfully to your email address."
                else:
                    # sms_response = send_otp_via_sms(mobile_or_email, otp)
                    sms_response = send_otp_via_fast2sms(mobile_or_email, otp)
                    if sms_response.get("status") == "error":
                        return Response({
                            "status": {
                                "type": "error",
                                "message": "Failed to send OTP via SMS. Please try again.",
                                "code": 500,
                                "error": True
                            }
                        }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
                    response_message = "OTP sent successfully to your mobile number."

                return Response({
                    "status": {
                        "type": "success",
                        "message": response_message,
                        "otp": otp,
                        "code": 200,
                        "error": False
                    },
                "user": {
                    "email": user.mobile_or_email,
                    "user_id": user.id
                },
                    "otp": otp
                }, status=status.HTTP_200_OK)

            return Response({
                "status": {
                    "type": "error",
                    "message": "No OTP entries found for this user.",
                    "code": 404,
                    "error": True
                }
            }, status=status.HTTP_404_NOT_FOUND)

        except User.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "message": "User not found.",
                    "code": 404,
                    "error": True
                }
            }, status=status.HTTP_404_NOT_FOUND)


class ResetPasswordView(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        mobile_or_email = request.data.get('mobile_or_email')
        new_password = request.data.get('new_password')
        otp_received = request.data.get('otp')

        User = get_user_model()

        try:
            user = User.objects.get(mobile_or_email=mobile_or_email)
            if getattr(user, 'login_method', None) == 2:
                return Response({
                    "status": {
                        "type": "error",
                        "message": "Users logged in via Google cannot reset their password manually.",
                        "code": 403,
                        "error": True
                    }
                }, status=status.HTTP_403_FORBIDDEN)
            otp_verification = OTPVerification.objects.get(user=user, otp=otp_received)

            if timezone.now() > otp_verification.otp_expires_at:
                return Response({
                    "status": {
                        "type": "error",
                        "message": "OTP has expired",
                        "code": 400,
                        "error": True
                    }
                }, status=status.HTTP_400_BAD_REQUEST)

            otp_verification.is_verified = True
            otp_verification.save()
            user.set_password(new_password)
            user.save()

            return Response({
                "status": {
                    "type": "success",
                    "message": "Reset Password Sent Successfully",
                    "code": 200,
                    "OTP":otp_received,
                    "error": False
                },
                
                "user": {
                    "email": user.mobile_or_email,
                    "user_id": user.id
                },
            }, status=status.HTTP_200_OK)

        except OTPVerification.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "message": "Invalid OTP",
                    "code": 400,
                    "error": True
                }
            }, status=status.HTTP_400_BAD_REQUEST)
        except User.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "message": "Email / Mobile not registered ",
                    "code": 400,
                    "error": True
                }
            }, status=status.HTTP_404_NOT_FOUND)


class ChangePasswordView(generics.GenericAPIView):
    authentication_classes = [JWTAuthentication]
    permission_classes = [IsAuthenticated]
    serializer_class = ChangePasswordSerializer

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        
        if serializer.is_valid():
            current_password = serializer.validated_data['current_password']
            new_password = serializer.validated_data['new_password']
            confirm_password = serializer.validated_data['confirm_password']
            user = request.user

            if getattr(user, 'login_method', None) == 2:
                return Response({
                    "status": {
                        "type": "error",
                        "message": "Users logged in via Google cannot change their password.",
                        "code": 403,
                        "error": True
                    }
                }, status=status.HTTP_403_FORBIDDEN)

            if not check_password(current_password, user.password):
                return Response({
                    "status": {
                        "type": "error",
                        "message": "Current password is incorrect",
                        "code": 400,
                        "error": True
                    }
                }, status=status.HTTP_400_BAD_REQUEST)

            if new_password != confirm_password:
                return Response({
                    "status": {
                        "type": "error",
                        "message": "New password and confirm password do not match",
                        "code": 400,
                        "error": True
                    }
                }, status=status.HTTP_400_BAD_REQUEST)

            user.set_password(new_password)
            user.save()

            return Response({
                "status": {
                    "type": "success",
                    "message": "Password changed successfully",
                    "code": 200,
                    "error": False
                },
                "data": {
                    "user": {
                        "user_id": user.id,
                        "status": "Password changed"
                    }
                }
            }, status=status.HTTP_200_OK)

        return Response({
            "status": {
                "type": "error",
                "message": "Invalid input",
                "code": 400,
                "error": True,
                "errors": serializer.errors
            }
        }, status=status.HTTP_400_BAD_REQUEST)


class SetNewPasswordView(APIView):
    authentication_classes = []
    permission_classes = [AllowAny]
    def post(self, request, *args, **kwargs):
        # Retrieve data from the request
        user_id = request.data.get('user_id')
        new_password = request.data.get('new_password')
        confirm_password = request.data.get('confirm_password')

        # Ensure all required fields are provided
        if not user_id or not new_password or not confirm_password:
            return Response({
                "status": {
                    "type": "error",
                    "message": "'user_id', 'new_password', and 'confirm_password' are required.",
                    "code": 400,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_400_BAD_REQUEST)

        # Check if new password and confirm password match
        if new_password != confirm_password:
            return Response({
                "status": {
                    "type": "error",
                    "message": "Passwords do not match.",
                    "code": 400,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_400_BAD_REQUEST)

        try:
            user = User.objects.get(id=user_id)
        except User.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "message": "User not found.",
                    "code": 404,
                    "error": True
                },
                "data": []
            }, status=status.HTTP_404_NOT_FOUND)

        # Set the new password
        user.set_password(new_password)
        user.save()

        # Return the success response
        return Response({
            "status": {
                "type": "success",
                "message": "Password reset successfully completed",
                "code": 200,
                "error": False
            }, 
                "user": {
                    "email": user.mobile_or_email,
                    "user_id": user.id
                },
            "data": [
                {
                    "status": "Password reset successfully"
                }
            ]
        }, status=status.HTTP_200_OK)
    
class LogoutView(generics.GenericAPIView):
    permission_classes = [IsAuthenticated]
    serializer_class = LogoutSerializer

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        refresh_token = serializer.validated_data['refresh_token']

        try:
            # Fetch the token from OutstandingToken
            outstanding_token = OutstandingToken.objects.get(token=refresh_token)
            # Blacklist the token
            BlacklistedToken.objects.create(token=outstanding_token)
            user = request.user

            # Use HTTP_200_OK for a clear response
            return Response({
                "status": {
                    "type": "success",
                    "message": "Successfully logged out",
                    "code": 200,
                    "error": False
                }, 
                "user": {
                    "email": user.mobile_or_email,
                    "user_id": user.id
                },
            }, status=status.HTTP_200_OK)

        except OutstandingToken.DoesNotExist:
            return Response({
                "status": {
                    "type": "error",
                    "message": "Token not found or already blacklisted",
                    "code": 400,
                    "error": True
                }
            }, status=status.HTTP_400_BAD_REQUEST)
        except Exception as e:
            return Response({
                "status": {
                    "type": "error",
                    "message": f"Unexpected error occurred: {str(e)}",
                    "code": 500,
                    "error": True
                }
            }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)
class UserSearchView(APIView):
    permission_classes = [IsAuthenticated]  # Optional: restrict access to authenticated users
    authentication_classes = [JWTAuthentication]  # Optional: use JWT authentication

    def get(self, request):
        # Get search parameters
        query = request.query_params.get('query', '')  # Search term (e.g., mobile_or_email or username)
        filters = request.query_params.get('filters', None)  # Optional additional filters
        
        # Base queryset
        queryset = CustomUser.objects.all()
        
        # Apply search query to relevant fields
        if query:
            queryset = queryset.filter(
                Q(mobile_or_email__icontains=query) |
                Q(username__icontains=query)
            )
        
        # Apply additional filters (optional)
        if filters:
            if 'is_active' in filters:
                is_active = request.query_params.get('is_active', None)
                if is_active is not None:
                    queryset = queryset.filter(is_active=is_active == 'true')
            if 'is_verified' in filters:
                is_verified = request.query_params.get('is_verified', None)
                if is_verified is not None:
                    queryset = queryset.filter(is_verified=is_verified == 'true')

        # Check if the queryset is empty
        if not queryset.exists():
            return Response({"message": "User not found"}, status=404)

        # Serialize and return results
        serializer = RegisterSerializer(queryset, many=True)
        return Response(serializer.data, status=200) 


class StateListView(APIView):
    def get(self, request):
        states = State.objects.all()
        serializer = StateSerializer(states, many=True)
        return Response({"status": "success", "data": serializer.data})

class CityListView(APIView):
    def post(self, request):
        state_id = request.data.get('state_id')
        if not state_id:
            return Response({"status": "error", "message": "State ID is required"}, status=400)
        
        cities = City.objects.filter(state_id=state_id)
        serializer = CitySerializer(cities, many=True)
        return Response({"status": "success", "data": serializer.data})
    
class StateShortNamesListView(APIView):
    def get(self, request):
        states = StateShortNames.objects.all()
        serializer = StateShortNamesSerializer(states, many=True)
        return Response({"status": "success", "data": serializer.data})
    
class LoggedDevicesView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        devices = DeviceList.objects.filter(user=user).values(
            "session_id",
            "device_name",
            "device_ip",
            "login_time",
            "status"
        )
        return Response(devices, status=status.HTTP_200_OK)

# class LogoutView(APIView):
#     permission_classes = [IsAuthenticated]  # Ensure the user is authenticated

#     def post(self, request):
#         # Get the session ID from the request
#         session_id = request.data.get("session_id")

#         if not session_id:
#             return Response({
#                 "status": {
#                     "type": "failed",
#                     "message": "Session ID is required",
#                     "code": 400,
#                     "error": True
#                 },
#                 "data": []
#             }, status=status.HTTP_400_BAD_REQUEST)

#         try:
#             # Attempt to delete the session record by session_id
#             device = DeviceList.objects.get(session_id=session_id, user=request.user)
#             device.delete()  # Deletes the record

#             return Response({
#                 "status": {
#                     "type": "success",
#                     "message": "Logged out successfully",
#                     "code": 200,
#                     "error": False
#                 },
#                 "data": []
#             }, status=status.HTTP_200_OK)

#         except DeviceList.DoesNotExist:
#             return Response({
#                 "status": {
#                     "type": "failed",
#                     "message": "Session not found",
#                     "code": 404,
#                     "error": True
#                 },
#                 "data": []
#             }, status=status.HTTP_404_NOT_FOUND)

class LogoutView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        session_id = request.data.get("session_id")
        if not session_id:
            return Response({
                "status": {
                    "type": "failed",
                    "message": "Session ID is required.",
                    "code": 400,
                    "error": True
                }
            }, status=400)

        try:
            # Find the device associated with the session
            device = DeviceList.objects.get(session_id=session_id, user=request.user, status="active")
            device.status = "logged_out"
            device.save()

            # Invalidate the token (optional for Single Token per Device logic)
            try:
                print(session_id)
                token = OutstandingToken.objects.get(token=session_id)
                print(token)
                BlacklistedToken.objects.create(token=token)
            except OutstandingToken.DoesNotExist:
                return Response({
                "status": {
                    "type": "failed",
                    "message": "Outstanding token not found",
                    "code": 404,
                    "error": True
                }
            }, status=404)

            return Response({
                "status": {
                    "type": "success",
                    "message": "Logged out successfully.",
                    "code": 200,
                    "error": False
                }
            }, status=200)
        except DeviceList.DoesNotExist:
            return Response({
                "status": {
                    "type": "failed",
                    "message": "Active session not found.",
                    "code": 404,
                    "error": True
                }
            }, status=404)

class ChangeEmailRequestView(APIView):
    def post(self, request):
        try:
            custom_user = CustomUser.objects.get(id= request.user.id)
        except CustomUser.DoesNotExist:
            return Response({"error": "User does not exist."}, status=status.HTTP_404_NOT_FOUND)
        if custom_user.login_method != '1':
            return Response({
                "error": "You are not allowed to change your email address with this login method."
            }, status=status.HTTP_403_FORBIDDEN)
            
        new_email = request.data.get("new_email")
        if not new_email:
            return Response({"error": "New email address is required."}, status=status.HTTP_400_BAD_REQUEST)
        if CustomUser.objects.filter(mobile_or_email=new_email).exists():
            return Response({"error": "Email address is already in use."}, status=status.HTTP_400_BAD_REQUEST)
        
        otp = str(random.randint(100000, 999999))
        print(otp)
        otp_verification, created = OTPVerification.objects.update_or_create(
            user=request.user,
            defaults={"otp": otp, "otp_expires_at": timezone.now() + timezone.timedelta(minutes=10), "is_verified": False},
        )

        send_mail(
            subject="Your OTP for Email Change",
            message=f"Your OTP is: {otp}",
            from_email="noreply@example.com",
            recipient_list=[new_email],
        )

        return Response({"message": "OTP sent to the new email address."}, status=status.HTTP_200_OK)
    
class VerifyEmailView(APIView):
    def post(self, request):
        otp = request.data.get("otp")
        new_email = request.data.get("new_email")

        if not otp or not new_email:
            return Response({"error": "OTP and new email address are required."}, status=status.HTTP_400_BAD_REQUEST)

        try:
            otp_verification = OTPVerification.objects.get(user=request.user, otp=otp, is_verified=False)
        except OTPVerification.DoesNotExist:
            return Response({"error": "Invalid or expired OTP."}, status=status.HTTP_400_BAD_REQUEST)

        # Check OTP expiry
        if otp_verification.otp_expires_at < timezone.now():
            return Response({"error": "OTP has expired."}, status=status.HTTP_400_BAD_REQUEST)

        # Update user's email
        request.user.mobile_or_email = new_email
        request.user.save()

        # Mark OTP as verified
        otp_verification.is_verified = True
        otp_verification.save()

        return Response({"message": "Email address updated successfully."}, status=status.HTTP_200_OK)
    
class DeleteAccountView(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, request):
        user = request.user
        reason = request.data.get('why_deleting')
        password = request.data.get('password')

        if not reason:
            return Response({
                "status": "error",
                "message": "Reason for deleting account is required."
            }, status=status.HTTP_400_BAD_REQUEST)
        
        if not password:
            return Response({
                "status": "error",
                "message": "Password is required to delete the account."
            }, status=status.HTTP_400_BAD_REQUEST)

        # Validate the provided password
        user = authenticate(mobile_or_email=user.mobile_or_email, password=password)
        if not user:
            return Response({
                "status": "error",
                "message": "Invalid password. Please provide the correct password to delete the account."
            }, status=status.HTTP_401_UNAUTHORIZED)

        user.is_active = False
        user.save()
        DeletedAccount.objects.create(user=user, reason=reason)

        return Response({
            "status": "success",
            "message": "Account deleted successfully."
        }, status=status.HTTP_200_OK)
    
class UserVerificationStatus(APIView):
    permission_classes = [AllowAny]

    def get(self, request, user_id):
        try:
            # Get user instance by ID
            user = CustomUser.objects.get(id=user_id)
            
            email_verified = user.is_verified
            mobile_verified = user.is_mobile_verified

            # Construct response data
            data = {
                "email_verified": email_verified,
                "mobile_verified": mobile_verified
            }

            return Response(data, status=status.HTTP_200_OK)

        except CustomUser.DoesNotExist:
            return Response({
                "error": "User not found"
            }, status=status.HTTP_404_NOT_FOUND)
        
class MobileNummberVerification(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        user_id = request.data.get('user_id')
        # user = request.user
        mobile = request.data.get('mobile')

        if CustomUser.objects.filter(mobile_number=mobile).exclude(id=user_id).exists():
            return Response({"error": "Mobile number already exists in the system"}, status=status.HTTP_400_BAD_REQUEST)

        try:
            user = CustomUser.objects.get(id=user_id)
            user.mobile_number = mobile
            user.save()

            # Generate OTP
            otp = str(random.randint(100000, 999999))
            print(otp)

            # Save OTP to the OTPVerification model
            OTPVerification.objects.create(
                user=user,
                otp=otp,
                otp_expires_at=timezone.now() + timedelta(minutes=10)
            )

            # Send OTP via SMS
            # response_data = send_otp_via_sms(mobile, otp)
            response_data = send_otp_via_fast2sms(mobile, otp)

            if response_data.get('status') == 'success':
                return Response({"message": f"OTP {otp} sent to your mobile number"}, status=status.HTTP_200_OK)
            else:
                return Response({
                    "status": "failed",
                    "message": "Failed to send OTP via SMS",
                    "code": 500
                }, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

        except CustomUser.DoesNotExist:
            return Response({"error": "User not found"}, status=status.HTTP_404_NOT_FOUND)


# Email Store and Verification API
class EmailIDVerification(APIView):
    permission_classes = [AllowAny]

    def post(self, request):
        user_id = request.data.get('user_id')
        # user = request.user
        email = request.data.get('email')

        if CustomUser.objects.filter(mobile_or_email=email).exclude(id=user_id).exists():
            return Response({"error": "Email already exists in the system"}, status=status.HTTP_400_BAD_REQUEST)

        try:
            user = CustomUser.objects.get(id=user_id)

            # Store email in the user model
            user.mobile_or_email = email
            user.is_verified = False
            user.save()

            # Generate OTP
            otp = str(random.randint(100000, 999999))
            print(otp)
            # Save OTP to OTPVerification model
            OTPVerification.objects.create(
                user=user,
                otp=otp,
                otp_expires_at=timezone.now() + timedelta(minutes=10)
            )

            # Send OTP via email
            send_mail(
                subject="Your OTP Code",
                message=f'Your OTP code is {otp}. It is valid for 10 minutes.',
                from_email=settings.EMAIL_HOST_USER,
                recipient_list=[email]
            )

            return Response({"message": f"OTP {otp} sent to your email address"}, status=status.HTTP_200_OK)

        except CustomUser.DoesNotExist:
            return Response({"error": "User not found"}, status=status.HTTP_404_NOT_FOUND)
        
class CountryNamesView(APIView):
    def get(self, request):
        country_names = Country.objects.values_list("name", flat=True)
        return Response({"countries": list(country_names)})


@method_decorator(csrf_exempt, name="dispatch")
class GenerateUserTokenView(View):
    def get(self, request):
        """Generates a user token for the Agora service."""
        user_id = request.GET.get("account")
        expire = int(request.GET.get("expireTimeInSeconds", 315360000))

        if not settings.AGORA_APP_ID or not settings.AGORA_APP_CERTIFICATE:
            return JsonResponse({"error": "APP_ID or APP_CERTIFICATE is not set"}, status=400)
        if not user_id:
            return JsonResponse({"error": "Missing 'account' parameter"}, status=400)

        # token = ChatTokenBuilder.build_user_token(settings.AGORA_APP_ID, settings.AGORA_APP_CERTIFICATE, user_id, expire=settings.AGORA_EXPIRE_TIME)
        token = ChatTokenBuilder.build_user_token(APP_ID, APP_CERTIFICATE, user_id, expire=settings.AGORA_EXPIRE_TIME)
        return JsonResponse({"token": token})


@method_decorator(csrf_exempt, name="dispatch")
class GenerateAppTokenView(View):
    def get(self, request):
        """Generates an app token for the Agora service."""
        expire = int(request.GET.get("expireTimeInSeconds", 315360000))

        if not APP_ID or not APP_CERTIFICATE:
            return JsonResponse({"error": "APP_ID or APP_CERTIFICATE is not set"}, status=400)

        # token = ChatTokenBuilder.build_app_token(settings.AGORA_APP_ID, settings.AGORA_APP_CERTIFICATE, expire=settings.AGORA_EXPIRE_TIME)
        token = ChatTokenBuilder.build_app_token(APP_ID, APP_CERTIFICATE, expire=settings.AGORA_EXPIRE_TIME)
        
        return JsonResponse({"token": token, "App_id":APP_ID,"App_certificate":APP_CERTIFICATE})
    
class PrivacySettingsView(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        user = request.user
        privacy_settings, created = PrivacySettings.objects.get_or_create(user=user)
        serializer = PrivacySettingsSerializer(privacy_settings)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def put(self, request):
        user = request.user
        privacy_settings, created = PrivacySettings.objects.get_or_create(user=user)
        serializer = PrivacySettingsSerializer(privacy_settings, data=request.data, partial=True)

        if serializer.is_valid():
            serializer.save()
            return Response({"message": "Privacy settings updated successfully", "data": serializer.data}, status=status.HTTP_200_OK)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    
class ChangeMobileRequestView(APIView):
    def post(self, request):
        try:
            custom_user = CustomUser.objects.get(id=request.user.id)
        except CustomUser.DoesNotExist:
            return Response({"error": "User does not exist."}, status=status.HTTP_404_NOT_FOUND)
        
        if custom_user.login_method != '1':
            return Response({
                "error": "You are not allowed to change your mobile number with this login method."
            }, status=status.HTTP_403_FORBIDDEN)
        
        new_mobile = request.data.get("new_mobile")
        if not new_mobile:
            return Response({"error": "New mobile number is required."}, status=status.HTTP_400_BAD_REQUEST)
        
        if CustomUser.objects.filter(mobile_number = new_mobile).exists():
            return Response({"error": "Mobile number is already in use."}, status=status.HTTP_400_BAD_REQUEST)
        
        otp = str(random.randint(100000, 999999))
        print(otp)
        otp_verification, created = OTPVerification.objects.update_or_create(
            user=request.user,
            defaults={"otp": otp, "otp_expires_at": timezone.now() + timezone.timedelta(minutes=10), "is_verified": False},
        )
        
        response_data = send_otp_via_fast2sms(new_mobile, otp)
        if response_data.get('status') == 'success':
            return Response({"message": "OTP sent to the new mobile number."}, status=status.HTTP_200_OK)
        else:
            return Response({"error": "Failed to send OTP."}, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class VerifyMobileView(APIView):
    def post(self, request):
        otp = request.data.get("otp")
        new_mobile = request.data.get("new_mobile")
        
        if not otp or not new_mobile:
            return Response({"error": "OTP and new mobile number are required."}, status=status.HTTP_400_BAD_REQUEST)
        
        try:
            otp_verification = OTPVerification.objects.get(user=request.user, otp=otp, is_verified=False)
        except OTPVerification.DoesNotExist:
            return Response({"error": "Invalid or expired OTP."}, status=status.HTTP_400_BAD_REQUEST)
        
        if otp_verification.otp_expires_at < timezone.now():
            return Response({"error": "OTP has expired."}, status=status.HTTP_400_BAD_REQUEST)
        
        request.user.mobile_number = new_mobile
        request.user.save()
        
        otp_verification.is_verified = True
        otp_verification.save()
        
        return Response({"message": "Mobile number updated successfully."}, status=status.HTTP_200_OK)